/*
  - Schuelerprojektwoche - Universität Kassel 2020

  Mit dieser Datei sollen die Funktionen der gelöteten Platine getestet werden, damit
  sichergestellt ist, dass im weiteren Verlauf alle Funktionen verwendet werden können.
  
*/

// constants won't change. They're used here to set pin numbers:
const int buttonPin = D8; // the number of the pushbutton pin
const int ledPinG =  D5;  // the number of the green LED pin
const int ledPinY =  D6;  // the number of the yellow LED pin
const int ledPinR =  D7;  // the number of the red LED 
const int sensorPinRead = A0; //moisture Sensor input
const int sensorPinPower = D1; //moisture Sensor power pin
const int pumpPin = D3; //pump pin

// Variables will change:
int ledStateG = LOW;             // ledState used to set the green LED
int ledStateY = LOW;             // ledState used to set the yellow LED
int ledStateR = LOW;             // ledState used to set the red LED
int switchState = LOW;         // the current state of the output pin
int pumpState = LOW;          //the current state of the pump
int buttonState;             // the current reading from the input pin
int lastButtonState = LOW;   // the previous reading from the input pin

int sensorValue = 0;  //intial sensor value
int showCounter = 0;  //counter for the light show

// the following variables are unsigned longs because the time, measured in
// milliseconds, will quickly become a bigger number than can be stored in an int.
unsigned long lastDebounceTime = 0;  // the last time the output pin was toggled
unsigned long lastSwitchTime = 0;   // the last time the switchState was HIGH
unsigned long debounceDelay = 50;    // the debounce time; increase if the output flickers

// the setup routine runs once when you press reset:
void setup() {
  
  pinMode(buttonPin, INPUT);  //define button pin as an input
  pinMode(ledPinG, OUTPUT);   //define green led pin as an output
  pinMode(ledPinY, OUTPUT);   //define yellow led pin as an output
  pinMode(ledPinR, OUTPUT);   //define red led pin as an output
  pinMode(sensorPinRead, INPUT);  //define pin for reading the sensor value as an input
  pinMode(sensorPinPower, OUTPUT);  //define the pin for powering the sensor as an output
  pinMode(pumpPin, OUTPUT);   //define pin for powering the pump as an output

  Serial.begin(115200); //start serial output with speed 115200 Hz
}

void loop() {
  
  // read the state of the switch into a local variable:
  int reading = digitalRead(buttonPin);

  // check to see if you just pressed the button
  // (i.e. the input went from LOW to HIGH), and you've waited long enough
  // since the last press to ignore any noise:

  // If the switch changed, due to noise or pressing:
  if (reading != lastButtonState) {
    // reset the debouncing timer
    lastDebounceTime = millis();
  }

  if ((millis() - lastDebounceTime) > debounceDelay) {
    // whatever the reading is at, it's been there for longer than the debounce
    // delay, so take it as the actual current state:

    // if the button state has changed:
    if (reading != buttonState) {
      buttonState = reading;

      // only toggle the switch state if the new button state is HIGH
      if (buttonState == HIGH) {
        switchState = !switchState;
      }
    }
  }

  /**********************************************************************************************************
   * Switchable part
   * ********************************************************************************************************/
  // switch is on
  if (switchState == HIGH){

    //light show
    //if the light show was not shown yet
    if (showCounter == 0){
          
      //Initial light show
      lastSwitchTime = millis();  //save actual time stamp
      //green led for 200ms
      while(millis() - lastSwitchTime < 200){
        digitalWrite(ledPinG, HIGH);  //switch green led on
      }    
      digitalWrite(ledPinG,LOW);    //switch green led off

      //yellow led for 200ms
      lastSwitchTime = millis();  //save actual time stamp
      while(millis() - lastSwitchTime < 200){
        digitalWrite(ledPinY, HIGH);  //switch yellow led on
      }
      digitalWrite(ledPinY, LOW);   //switch yellow led off
  
      lastSwitchTime = millis(); //save actual time stamp
      while(millis() - lastSwitchTime < 200){
        digitalWrite(ledPinR, HIGH);  //switch red led on
      }
      digitalWrite(ledPinR, LOW);   //switch red led off

      showCounter++;  //increment showCounter
    }

    //sensor measurement
    digitalWrite(sensorPinPower, HIGH); //switch sensor on
    sensorValue = analogRead(sensorPinRead);  //take measurement from the sensor
    digitalWrite(sensorPinPower, LOW);  //switch sensor on

    Serial.println(sensorValue);  //print sensor value to terminal

    //if sensor valur is between 0 and 100
    if (sensorValue > 0 && sensorValue <= 100){
      ledStateR = HIGH;   //state: red led on
      ledStateY = LOW;    //state: yellow led off
      ledStateG = LOW;    //state: green led off 
      pumpState = HIGH;   //state: pump on
    }
    //if sensor value is betweeen 100 and 300
    else if(sensorValue > 100 && sensorValue <= 300){
      ledStateR = LOW;    //state: red led off
      ledStateY = HIGH;   //state: yellow led on
      ledStateG = LOW;    //state: green led off
      pumpState = LOW;    //state: pump off
    }
    //if sensor value is above 300
    else if(sensorValue > 300){
      ledStateR = LOW;    //state: red led off
      ledStateY = LOW;    //state: yellow led off
      ledStateG = HIGH;   //state: green led on
      pumpState = LOW;    //state: pump off
    }
    
  }
  // switch is off
  else if (switchState == LOW){
    ledStateG = LOW;    //state: red led off
    ledStateY = LOW;    //state: yellow led off
    ledStateR = LOW;    //state: green led off
    pumpState = LOW;    //state: pump off

    showCounter = 0;  //restart light show
  }

  digitalWrite(ledPinG, ledStateG); //set green led pin to the corresponding state
  digitalWrite(ledPinY, ledStateY); //set yellow led pin to the corresponding state
  digitalWrite(ledPinR, ledStateR); //set red led pin to the corresponding state
  digitalWrite(pumpPin, pumpState); //set pump pin to the corresponding state

  /**********************************************************************************************************/

  // save the reading. Next time through the loop, it'll be the lastButtonState:
  lastButtonState = reading;
}
